/*
    Copyright (C) 2012-2014 Carl Hetherington <cth@carlh.net>

    This file is part of libdcp.

    libdcp is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    libdcp is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libdcp.  If not, see <http://www.gnu.org/licenses/>.

    In addition, as a special exception, the copyright holders give
    permission to link the code of portions of this program with the
    OpenSSL library under certain conditions as described in each
    individual source file, and distribute linked combinations
    including the two.

    You must obey the GNU General Public License in all respects
    for all of the code used other than OpenSSL.  If you modify
    file(s) with this exception, you may extend this exception to your
    version of the file(s), but you are not obligated to do so.  If you
    do not wish to do so, delete this exception statement from your
    version.  If you delete this exception statement from all source
    files in the program, then also delete it here.
*/

/** @file  src/picture_asset_writer.h
 *  @brief PictureAssetWriter and FrameInfo classes.
 */

#ifndef LIBDCP_PICTURE_ASSET_WRITER_H
#define LIBDCP_PICTURE_ASSET_WRITER_H

#include "metadata.h"
#include "types.h"
#include "asset_writer.h"
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>
#include <stdint.h>
#include <string>

namespace dcp {

class PictureAsset;

/** @class FrameInfo
 *  @brief Information about a single frame (either a monoscopic frame or a left *or* right eye stereoscopic frame)
 */
struct FrameInfo
{
	FrameInfo ()
		: offset (0)
		, size (0)
	{}

	FrameInfo (uint64_t o, uint64_t s, std::string h)
		: offset (o)
		, size (s)
		, hash (h)
	{}

	uint64_t offset;
	uint64_t size;
	std::string hash;
};

/** @class PictureAssetWriter
 *  @brief Parent class for classes which write picture assets.
 */
class PictureAssetWriter : public AssetWriter
{
public:
	virtual FrameInfo write (uint8_t const *, int) = 0;
	virtual void fake_write (int) = 0;

protected:
	template <class P, class Q>
	friend void start (PictureAssetWriter *, boost::shared_ptr<P>, Standard, Q *, uint8_t const *, int);

	PictureAssetWriter (PictureAsset *, boost::filesystem::path, Standard standard, bool);

	PictureAsset* _picture_asset;
	Standard _standard;
	bool _overwrite;
};

}

#endif
